dnl ---------------------------------------------------------------------------
dnl Check REXX library and header files
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_REXX],
[
if test "$with_regina" = yes; then
	AC_DEFINE(USE_REGINA)
	rexx_h="rexxsaa.h"
	rexx_l="regina"
	extra_libs=""
	REXX_INT="Regina"
else
	if test "$with_rexximc" = yes; then
		AC_DEFINE(USE_REXXIMC)
		rexx_h="rexxsaa.h"
		rexx_l="rexx"
		extra_libs=""
		REXX_INT="REXX/imc"
	else
		if test "$with_rexx6000" = yes; then
			AC_DEFINE(USE_REXX6000)
			rexx_h="rexxtype.h"
			rexx_l="rexx"
			extra_libs=""
			REXX_INT="REXX/6000"
		else
			if test "$with_orexx" = yes; then
				AC_DEFINE(USE_OREXX)
				rexx_h="rexx.h"
				rexx_l="rexx"
				case "$target" in
					*ibm-aix*)
					extra_libs="-lrexxapi -lrexx"
					;;
					*linux*)
					extra_libs="-lrexxapi -lpthreads"
					;;
				esac
				REXX_INT="Object Rexx"
			else
				if test "$with_unirexx" = yes; then
					AC_DEFINE(USE_UNIREXX)
					rexx_h="rexxsaa.h"
					rexx_l="rx"
					extra_libs=""
					REXX_INT="uni-REXX"
				else
					AC_DEFINE(NOREXX)
					rexx_h=""
					rexx_l=""
					extra_libs=""
				fi
			fi
		fi
	fi
fi

dnl look for REXX header and library, exit if not found

if test "xx$rexx_h" = "xx" ; then
	MH_REXX_LIB=""
	MH_REXX_INC=""
	AC_SUBST(MH_REXX_INC)
	AC_SUBST(MH_REXX_LIB)
else
dnl look for REXX header and library, exit if not found
	AC_MSG_CHECKING(for location of REXX header file)
	mh_rexx_inc_dir=""
	mh_inc_dirs="\
	    ${HOME}/include           \
	    /usr/local/include        \
	    /usr/contrib/include      \
	    /opt/include              \
	    /usr/include              \
	    /usr/local/orexx          \
	    /usr/unsupported/include"
dnl
dnl Provide for user supplying directory
dnl
	if test "$with_rexxincdir" != no ; then
		mh_inc_dirs="$with_rexxincdir $mh_inc_dirs"
	fi
dnl
dnl Try to determine the directory containing REXX header
dnl
	for ac_dir in $mh_inc_dirs ; do
	  if test -r $ac_dir/$rexx_h; then
	    mh_rexx_inc_dir=$ac_dir
	    break
	  fi
	done
	if test "x$mh_rexx_inc_dir" != "x" ; then
		MH_REXX_INC="-I$mh_rexx_inc_dir"
		AC_MSG_RESULT(found in $mh_rexx_inc_dir)
		AC_SUBST(MH_REXX_INC)
	else
		AC_MSG_ERROR(Cannot find REXX header file: $rexx_h; THE cannot be configured)
	fi
	AC_MSG_CHECKING(for location of REXX library file)
	mh_rexx_lib_dir=""
	mh_lib_dirs="\
	    ${HOME}/lib               \
	    /usr/local/lib            \
	    /usr/contrib/lib          \
	    /opt/lib                  \
	    /usr/lib                  \
	    /usr/local/orexx/lib      \
	    /usr/unsupported/lib"
dnl
dnl Provide for user supplying directory
dnl
	if test "$with_rexxlibdir" != no ; then
		mh_lib_dirs="$with_rexxlibdir $mh_lib_dirs"
	fi
dnl
dnl Try to determine the directory containing REXX library
dnl
	for ac_dir in $mh_lib_dirs ; do
		for mh_ext in lib${rexx_l}.a lib${rexx_l}.so lib${rexx_l}.sl ${rexx_l}.lib; do
		  if test -r $ac_dir/$mh_ext; then
		     mh_rexx_lib_dir=$ac_dir
		     break 2
		  fi
		done
	done
	if test "x$mh_rexx_lib_dir" != "x" ; then
		MH_REXX_LIB="-L$mh_rexx_lib_dir -l$rexx_l $extra_libs"
		AC_MSG_RESULT(found in $mh_rexx_lib_dir)
		AC_SUBST(MH_REXX_LIB)
	else
		AC_MSG_ERROR(Cannot find REXX library file: $rexx_l; Rexx/SQL cannot be configured)
	fi
fi
])dnl
dnl ---------------------------------------------------------------------------
dnl Check if C compiler supports -c -o
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_CC_O],
[
AC_MSG_CHECKING(whether $CC understand -c and -o together)
set dummy $CC; ac_cc="`echo [$]2 |
changequote(, )dnl
		       sed -e 's/[^a-zA-Z0-9_]/_/g' -e 's/^[0-9]/_/'`"
changequote([, ])dnl
AC_CACHE_VAL(ac_cv_prog_cc_${ac_cc}_c_o,
[echo 'foo(){}' > conftest.c
# We do the test twice because some compilers refuse to overwrite an
# existing .o file with -o, though they will create one.
eval ac_cv_prog_cc_${ac_cc}_c_o=no
ac_try='${CC-cc} -c conftest.c -o conftest.ooo 1>&AC_FD_CC'
if AC_TRY_EVAL(ac_try) && test -f conftest.ooo && AC_TRY_EVAL(ac_try);
then
  ac_try='${CC-cc} -c conftest.c -o conftest.ooo 1>&AC_FD_CC'
  if AC_TRY_EVAL(ac_try) && test -f conftest.ooo && AC_TRY_EVAL(ac_try);
  then
    eval ac_cv_prog_cc_${ac_cc}_c_o=yes
  fi
fi
rm -f conftest*
])dnl
if eval "test \"`echo '$ac_cv_prog_cc_'${ac_cc}_c_o`\" = yes"; then
	OHASH="echo ignoring command -"
	CC2O="-o $"'@'
	AC_MSG_RESULT(yes)
else
	OHASH=""
	CC2O=""
	AC_MSG_RESULT(no)
fi
AC_SUBST(OHASH)
AC_SUBST(CC2O)
])

dnl ---------------------------------------------------------------------------
dnl Check if various types have been typedefed by database header files
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_TYPE],
[
dnl
dnl override existing value of $ac_compile so we use the correct compiler
dnl SHOULD NOT NEED THIS
dnl
ac_compile='$ac_cv_prog_CC conftest.$ac_ext $MH_DB_INC $CFLAGS $CPPFLAGS -c -o conftest.o 1>&5 2>&5'
for mh_type in $1; do
	AC_MSG_CHECKING(if $mh_type defined)
AC_TRY_COMPILE([#include <${DBINC}>],
[${mh_type} var1;],
  mh_type_found=yes, mh_type_found=no )
	AC_MSG_RESULT($mh_type_found)
	if test "$mh_type_found" = yes; then
	AC_DEFINE_UNQUOTED(${mh_type}_TYPEDEFED)
	fi
done
])dnl

dnl ---------------------------------------------------------------------------
dnl Check for where yywrap lives
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_LEX],
[
MHLEXLIB=""
for mh_lib in $1; do
	AC_CHECK_LIB($mh_lib,yywrap,mh_lib_found=yes,mh_lib_found=no)
	if test "$mh_lib_found" = yes; then
		MHLEXLIB="-l$mh_lib"
		break
	fi
done
if test "$MHLEXLIB" = ""; then
	AC_MSG_ERROR(Cannot find a library that contains yywrap; unable to build Rexx/SQL with Regina)
fi
])dnl

dnl ---------------------------------------------------------------------------
dnl Check for presense of various libraries
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_LIB],
[
MH_EXTRA_LIBS=''
for mh_lib in $1; do
#	AC_MSG_CHECKING(for library -l${mh_lib})
	AC_CHECK_LIB($mh_lib,main,mh_lib_found=yes,mh_lib_found=no)
	if test "$mh_lib_found" = yes; then
#		AC_MSG_RESULT(found)
		MH_EXTRA_LIBS="${MH_EXTRA_LIBS} -l${mh_lib}"
#	else
#		AC_MSG_RESULT(not found)
	fi
done
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all Oracle stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_ORACLE],
[
DBINC="oci.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="Oracle"
INCLUDE_DB_MAKEFILE=""
if test -n "$ORACLE_HOME" ; then
	if test -f "$ORACLE_HOME/lib/libclntsh.$SHL"; then
		DBLIB1="-L$ORACLE_HOME/lib -lclntsh"
		mh_db_lib_dir="$ORACLE_HOME/lib"
		AC_MSG_RESULT(using Oracle client library -lclntsh)
	else
		grep libclntsh $ORACLE_HOME/rdbms/lib/*.mk > /dev/null
		if test $? -eq 0 ; then
			AC_MSG_WARN(This version of Oracle supports a shared library of client code)
			AC_MSG_WARN(used by OCI programs.  The shared library; libclntsh.$SHL in)
			AC_MSG_WARN($ORACLE_HOME/lib has not been built.  See your DBA to build this)
			AC_MSG_WARN(library and then run this configuration script again.)
			AC_MSG_WARN(You will need to include $ORACLE_HOME/lib in your LD_LIBRARY_PATH.)
			AC_MSG_ERROR(configure aborted.)
		else
			if test -f "$ORACLE_HOME/rdbms/lib/oracle.mk" ; then
				INCLUDE_DB_MAKEFILE="include $ORACLE_HOME/rdbms/lib/oracle.mk"
				DBLIB1="-L$ORACLE_HOME/lib $(OCILDLIBS)"
				mh_db_lib_dir="$ORACLE_HOME/lib"
				AC_MSG_RESULT(using $ORACLE_HOME/rdbms/lib/oracle.mk)
			else
				AC_MSG_WARN(Unable to configure Rexx/SQL for Oracle.)
				AC_MSG_WARN(Rexx/SQL needs one of the following files to be available:)
				AC_MSG_WARN()
				AC_MSG_WARN($ORACLE_HOME/lib/libclntsh.$SHL)
				AC_MSG_WARN($ORACLE_HOME/rdbms/lib/oracle.mk)
				AC_MSG_WARN(or)
				AC_MSG_WARN(rules to make libclntsh.$SHL in)
				AC_MSG_WARN($ORACLE_HOME/rdbms/lib/*.mk)
			AC_MSG_ERROR(configure aborted.)
			fi
		fi
	fi
else
	AC_MSG_ERROR(ORACLE_HOME not set, cannot continue)
fi
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all Sybase stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_SYBASE],
[
DBINC="ctpublic.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="Sybase System 10/11"
AC_MSG_CHECKING(for Sybase headers and libraries)
if test -n "$SYBASE" ; then
	if test "$with_dbincdir" = no; then
		MH_DB_INC="-I${SYBASE}/include"
		mh_db_inc_dir="$SYBASE/include"
	else
		MH_DB_INC="-I${with_dbincdir}"
		mh_db_inc_dir="$with_dbincdir"
	fi
	mh_db_lib_dir="$SYBASE/lib"
	mh_syb_libraries="cs tcl comn intl sybtcl insck"
	if test "$with_dblibdir" = no; then
		if test -f "${SYBASE}/lib/libctlib.a"; then
			mh_syb_lib="-lctlib"
		else
			if test -f "${SYBASE}/lib/libct.a"; then
				mh_syb_lib="-lct"
			fi
		fi
		for a_lib in $mh_syb_libraries; do
			if test -f "${SYBASE}/lib/lib${a_lib}.a"; then
				mh_syb_lib="${mh_syb_lib} -l${a_lib}"
			fi
		done
		MH_DB_LIB="-L${SYBASE}/lib ${mh_syb_lib}"
	else
		if test -f "${with_dblibdir}/libctlib.a"; then
			mh_syb_lib="-lctlib"
		else
			if test -f "${with_dblibdir}/libct.a"; then
				mh_syb_lib="-lct"
			fi
		fi
		for a_lib in $mh_syb_libraries; do
			if test -f "${with_dblibdir}/lib${a_lib}.a"; then
				mh_syb_lib="${mh_syb_lib} -l${a_lib}"
			fi
		done
		MH_DB_LIB="-L${with_dblibdir} ${mh_syb_lib}"
	fi
	AC_MSG_RESULT(found)
else
	AC_MSG_ERROR(SYBASE not set, cannot continue)
fi
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all mSQL 1.x stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_MSQL],
[
DBINC="msql.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="miniSQL 1.x"
AC_MSG_CHECKING(location of mSQL header file)
if test "$with_dbincdir" = no; then
	if test -f "/usr/local/Minerva/include/msql.h"; then
		mh_db_inc_dir="/usr/local/Minerva/include"
	else
		AC_MSG_ERROR(msql.h not found in /usr/local/Minerva/include; you MUST specify directory where msql.h resides using --with-dbincdir=)
	fi
else
	if test -f "$with_dbincdir/msql.h"; then
		mh_db_inc_dir="${with_dbincdir}"
	else
		AC_MSG_ERROR(msql.h not found in $with_dbincdir)
	fi
fi
MH_DB_INC="-I$mh_db_inc_dir"
AC_MSG_RESULT(found in $mh_db_inc_dir)

AC_MSG_CHECKING(location of mSQL library file)
if test "$with_dblibdir" = no; then
	if test -f "/usr/local/Minerva/lib/libmsql.a"; then
		mh_db_lib_dir="/usr/local/Minerva/lib"
	else
		AC_MSG_ERROR(libmsql.a not found in /usr/local/Minerva/lib; you MUST specify directory where libmsql.a resides using --with-dblibdir=)
	fi
else
	if test -f "$with_dblibdir/libmsql.a"; then
		mh_db_lib_dir="${with_dblibdir}"
	else
		AC_MSG_ERROR(libmsql.a not found in $with_dblibdir)
	fi
fi
MH_DB_LIB="-L$mh_db_lib_dir -lmsql"
AC_MSG_RESULT(found in $mh_db_lib_dir)
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all mSQL 2.x stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_MSQL2],
[
DBINC="msql.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="miniSQL 2.x"
AC_MSG_CHECKING(location of mSQL header file)
if test "$with_dbincdir" = no; then
	if test -f "/usr/local/Hughes/include/msql.h"; then
		mh_db_inc_dir="/usr/local/Hughes/include"
	else
		AC_MSG_ERROR(msql.h not found in /usr/local/Hughes/include; you MUST specify directory where msql.h resides using --with-dbincdir=)
	fi
else
	if test -f "$with_dbincdir/msql.h"; then
		mh_db_inc_dir="${with_dbincdir}"
	else
		AC_MSG_ERROR(msql.h not found in $with_dbincdir)
	fi
fi
MH_DB_INC="-I$mh_db_inc_dir"
AC_MSG_RESULT(found in $mh_db_inc_dir)

AC_MSG_CHECKING(location of mSQL library file)
if test "$with_dblibdir" = no; then
	if test -f "/usr/local/Hughes/lib/libmsql.a"; then
		mh_db_lib_dir="/usr/local/Hughes/lib"
	else
		AC_MSG_ERROR(libmsql.a not found in /usr/local/Hughes/lib; you MUST specify directory where libmsql.a resides using --with-dblibdir=)
	fi
else
	if test -f "$with_dblibdir/libmsql.a"; then
		mh_db_lib_dir="${with_dblibdir}"
	else
		AC_MSG_ERROR(libmsql.a not found in $with_dblibdir)
	fi
fi
MH_DB_LIB="-L$mh_db_lib_dir -lmsql"
AC_MSG_RESULT(found in $mh_db_lib_dir)
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all mySQL stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_MYSQL],
[
DBINC="mysql.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="MySQL"
AC_MSG_CHECKING(location of mySQL header file)
if test "$with_dbincdir" = no; then
	if test -f "/usr/local/include/mysql/mysql.h"; then
		mh_db_inc_dir="/usr/local/include/mysql"
	else
		AC_MSG_ERROR(mysql.h not found in /usr/local/include/mysql; you MUST specify directory where mysql.h resides using --with-dbincdir=)
	fi
else
	if test -f "$with_dbincdir/mysql.h"; then
		mh_db_inc_dir="${with_dbincdir}"
	else
		AC_MSG_ERROR(mysql.h not found in $with_dbincdir)
	fi
fi
MH_DB_INC="-I$mh_db_inc_dir"
AC_MSG_RESULT(found in $mh_db_inc_dir)

AC_MSG_CHECKING(location of mySQL library file)
if test "$with_dblibdir" = no; then
	if test -f "/usr/local/lib/mysql/libmysqlclient.a"; then
		mh_db_lib_dir="/usr/local/lib/mysql"
	else
		AC_MSG_ERROR(libmysqlclient.a not found in /usr/local/lib/mysql; you MUST specify directory where libmysqlclient.a resides using --with-dblibdir=)
	fi
else
	if test -f "$with_dblibdir/libmysqlclient.a"; then
		mh_db_lib_dir="${with_dblibdir}"
	else
		AC_MSG_ERROR(libmysqlclient.a not found in $with_dblibdir)
	fi
fi
MH_DB_LIB="-L$mh_db_lib_dir -lmysqlclient"
AC_MSG_RESULT(found in $mh_db_lib_dir)
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all PostgreSQL ODBC stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_PGSQLODBC],
[
DBINC="isql.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="PostgreSQL ODBC"
AC_MSG_CHECKING(location of PostgreSQL ODBC header and library files)
if test "$with_dbincdir" = no; then
	AC_MSG_ERROR(you MUST specify directory where isql.h resides using --with-dbincdir=)
else
	if test -f "$with_dbincdir/isql.h"; then
		MH_DB_INC="-I${with_dbincdir}"
		mh_db_inc_dir="$with_dbincdir"
	else
		AC_MSG_ERROR(isql.h not found in $with_dbincdir)
	fi
fi
if test "$with_dblibdir" = no; then
	AC_MSG_ERROR(you MUST specify directory where libpsqlodbc.${SHL} or libpsqlodbc.a resides using --with-dblibdir=)
else
	if test -f "$with_dblibdir/libpsqlodbc.${SHL}" -o -f "$with_dblibdir/libpsqlodbc.a"; then
		MH_DB_LIB="-L${with_dblibdir} -lpsqlodbc"
		mh_db_lib_dir="$with_dblibdir"
	else
		AC_MSG_ERROR(libpsqlodbc.${SHL} not found in $with_dblibdir)
	fi
fi
AC_MSG_RESULT(found)
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all unixODBC ODBC stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_UNIXODBC],
[
DBINC="sqlext.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="unixODBC"
AC_MSG_CHECKING(location of unixODBC header and library files)
if test "$with_dbincdir" = no; then
	AC_MSG_ERROR(you MUST specify directory where sqlext.h resides using --with-dbincdir=)
else
	if test -f "$with_dbincdir/sqlext.h"; then
		MH_DB_INC="-I${with_dbincdir}"
		mh_db_inc_dir="$with_dbincdir"
	else
		AC_MSG_ERROR(sqlext.h not found in $with_dbincdir)
	fi
fi

if test "$with_dblibdir" = no; then
	AC_MSG_ERROR(you MUST specify directory where libodbc.${SHL} or libodbc.a resides using --with-dblibdir=)
else
	if test -f "$with_dblibdir/libodbc.${SHL}" -o -f "$with_dblibdir/libodbc.a"; then
		MH_DB_LIB="-L${with_dblibdir} -lodbc"
		mh_db_lib_dir="$with_dblibdir"
	else
		AC_MSG_ERROR(libodbc.${SHL} not found in $with_dblibdir)
	fi
fi
AC_MSG_RESULT(found)
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all easysoft ODBC-ODBC bridge stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_EASYSOFT_OOB],
[
DBINC="odbc.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="EasySoft ODBC-ODBC bridge"
AC_MSG_CHECKING(location of easysoft ODBC-ODBC bridge header file)
if test "$with_dbincdir" = no; then
	if test -f "/usr/local/easysoft/oob/client/include/odbc.h"; then
		mh_db_inc_dir="/usr/local/easysoft/oob/client/include"
	else
		AC_MSG_ERROR(odbcl.h not found in /usr/local/easysoft/oob/client/include; you MUST specify directory where odbc.h resides using --with-dbincdir=)
	fi
else
	if test -f "$with_dbincdir/odbc.h"; then
		mh_db_inc_dir="${with_dbincdir}"
	else
		AC_MSG_ERROR(odbc.h not found in $with_dbincdir)
	fi
fi
MH_DB_INC="-I$mh_db_inc_dir"
AC_MSG_RESULT(found in $mh_db_inc_dir)

AC_MSG_CHECKING(location of easysoft ODBC-ODBC bridge library file)
if test "$with_dblibdir" = no; then
	if test -f "/usr/local/easysoft/oob/client/lib/libesoobclient.${SHL}"; then
		mh_db_lib_dir="/usr/local/easysoft/oob/client/lib"
	else
		AC_MSG_ERROR(libesoobclient.${SHL} not found in /usr/local/easysoft/oob/client/lib; you MUST specify directory where libmysqlclient.a resides using --with-dblibdir=)
	fi
else
	if test -f "$with_dblibdir/libesoobclient.${SHL}"; then
		mh_db_lib_dir="${with_dblibdir}"
	else
		AC_MSG_ERROR(libesoobclient.${SHL} not found in $with_dblibdir)
	fi
fi
MH_DB_LIB="-L$mh_db_lib_dir -lesoobclient"
AC_MSG_RESULT(found in $mh_db_lib_dir)
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all Openlink ODBC stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_OPENLINK],
[
DBINC="isql.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="OpenLink ODBC"
AC_MSG_CHECKING(location of Openlink header and library files)
if test "$with_dbincdir" = no; then
	AC_MSG_ERROR(you MUST specify directory where isql.h resides using --with-dbincdir=)
else
	if test -f "$with_dbincdir/isql.h"; then
		MH_DB_INC="-I${with_dbincdir}"
		mh_db_inc_dir="$with_dbincdir"
	else
		AC_MSG_ERROR(isql.h not found in $with_dbincdir)
	fi
fi

if test "$with_dblibdir" = no; then
	AC_MSG_ERROR(you MUST specify directory where libiodbc.${SHL} or libiodbc.a resides using --with-dblibdir=)
else
	if test -f "$with_dblibdir/libiodbc.${SHL}" -o -f "$with_dblibdir/libiodbc.a"; then
		MH_DB_LIB="-L${with_dblibdir} -liodbc"
		mh_db_lib_dir="$with_dblibdir"
	else
		AC_MSG_ERROR(libiodbc.${SHL} not found in $with_dblibdir)
	fi
fi
AC_MSG_RESULT(found)
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all Openlink UDBC stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_OPENLINK_UDBC],
[
DBINC="libudbc.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="OpenLink UDBC"
AC_MSG_CHECKING(location of Openlink UDBC header and library files)
if test "$with_dbincdir" = no; then
	AC_MSG_ERROR(you MUST specify directory where libudbc.h resides using --with-dbincdir=)
else
	if test -f "$with_dbincdir/libudbc.h"; then
		MH_DB_INC="-I${with_dbincdir}"
		mh_db_inc_dir="$with_dbincdir"
	else
		AC_MSG_ERROR(libudbc.h not found in $with_dbincdir)
	fi
fi

if test "$with_dblibdir" = no; then
	AC_MSG_ERROR(you MUST specify directory where libudbc.${SHL} or libudbc.a resides using --with-dblibdir=)
else
	if test -f "$with_dblibdir/libudbc.${SHL}" -o -f "$with_dblibdir/libudbc.a"; then
		MH_DB_LIB="-L${with_dblibdir} -ludbc"
		mh_db_lib_dir="$with_dblibdir"
	else
		AC_MSG_ERROR(libudbc.${SHL} or libudbc.a not found in $with_dblibdir)
	fi
fi
AC_MSG_RESULT(found)
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all Solid ODBC stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_SOLID],
[
DBINC="cli0defs.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="Solid Server"
AC_MSG_CHECKING(location of Solid header and library files)
if test "$with_dbincdir" = no; then
	AC_MSG_ERROR(you MUST specify the directory where cli0defs.h resides using --with-dbincdir=)
else
	if test -f "$with_dbincdir/cli0defs.h"; then
		MH_DB_INC="-I${with_dbincdir}"
		mh_db_inc_dir="$with_dbincdir"
	else
		AC_MSG_ERROR(cli0defs.h not found in $with_dbincdir)
	fi
fi

if test "$with_dblibdir" = no; then
	AC_MSG_ERROR(you MUST specify the directory where scl${solid_os}nn.a {nn is version number} resides using --with-dblibdir=)
else
	if test -f "$with_dblibdir/libsolcli.${SHL}" -o -f "$with_dblibdir/libsolcli.a"; then
		MH_DB_LIB="-L${with_dblibdir} -lsolcli"
		mh_db_lib_dir="$with_dblibdir"
	else
		AC_MSG_ERROR(libsolcli.${SHL} or libsolcli.a not found in $with_dblibdir)
	fi
fi
AC_MSG_RESULT(found)
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all Velocis ODBC stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_VELOCIS],
[
DBINC="sqlext.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="Raima Velocis"
if test -n "$RDSHOME" ; then
	if test "$with_dbincdir" = no; then
		MH_DB_INC="-I${RDSHOME}/include"
		mh_db_inc_dir="$RDSHOME/include"
	else
		MH_DB_INC="-I${with_dbincdir}"
		mh_db_inc_dir="$with_dbincdir"
	fi

	if test "$with_dblibdir" = no; then
		MH_DB_LIB="-L${RDSHOME}/lib -lCadm -lCdict -lCenc -lCncp -lCrdbc -lCrdm -lCrm -lCrpc -lCuapi"
		mh_db_lib_dir="$RDSHOME/lib"
	else
		MH_DB_LIB="-L${with_dblibdir} -lCadm -lCdict -lCenc -lCncp -lCrdbc -lCrdm -lCrm -lCrpc -lCuapi"
		mh_db_lib_dir="$with_dblibdir"
	fi
else
	AC_MSG_ERROR(RDSHOME not set, cannot continue)
fi
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all SqlAnywhere stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_SQLANYWHERE],
[
DBINC="unixodbc.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="Sybase SQL Anywhere"
if test -n "$SQLANY" ; then
	if test "$with_dbincdir" = no; then
		MH_DB_INC="-I${SQLANY}/include"
		mh_db_inc_dir="$SQLANY/include"
	else
		MH_DB_INC="-I${with_dbincdir}"
		mh_db_inc_dir="$with_dbincdir"
	fi
	if test "$with_dblibdir" = no; then
		DBLIB1="${SQLANY}/lib/saodbc.so"
		mh_db_lib_dir="$SQLANY/lib"
	else
		DBLIB1="${with_dblibdir}/saodbc.so"
		mh_db_lib_dir="$with_dblibdir"
	fi
else
	AC_MSG_ERROR(SQLANY not set, cannot continue)
fi
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all DB2 stuff
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_CHECK_DB2],
[
DBINC="sqlcli.h"
DBLIB1=""
DBLIB2=""
DBLIB3=""
DBLIB4=""
mh_db_inc_dir=""
mh_db_lib_dir=""
mh_db_name="IBM DB2"
AC_MSG_CHECKING(location of DB2 header and library files)
if test "$with_dbincdir" = no; then
	AC_MSG_ERROR(you MUST specify the directory where sqlcli.h resides using --with-dbincdir=)
else
	if test -f "$with_dbincdir/sqlcli.h"; then
		MH_DB_INC="-I${with_dbincdir}"
		mh_db_inc_dir="$with_dbincdir"
	else
	AC_MSG_ERROR(sqlcli.h not found in $with_dbincdir)
	fi
fi

if test "$with_dblibdir" = no; then
	AC_MSG_ERROR(you MUST specify directory where libdb2ar.${SHL} or libdb2ar.a resides using --with-dblibdir=)
else
	if test -f "$with_dblibdir/libdb2ar.${SHL}" -o -f "$with_dblibdir/libdb2ar.a"; then
	MH_DB_LIB="-L${with_dblibdir} -ldb2ar"
		mh_db_lib_dir="$with_dblibdir"
	else
	AC_MSG_ERROR(libdb2ar.${SHL} or libdb2ar.a not found in $with_dblibdir)
	fi
fi
AC_MSG_RESULT(found)
])dnl

dnl ---------------------------------------------------------------------------
dnl Work out how to create a dynamically loaded module
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_HOWTO_DYN_LINK],
[
mh_compile='${CC-cc} -c $DYN_COMP conftest.$ac_ext 1>&AC_FD_CC'
cat > conftest.$ac_ext <<EOF
dnl [#]line __oline__ "[$]0"
[#]line __oline__ "configure"
int foo()
{
return(0);
}
EOF
if AC_TRY_EVAL(mh_compile) && test -s conftest.o; then
	mh_dyn_link='ld -shared -o conftest.rxlib conftest.o -lc 1>&AC_FD_CC'
	if AC_TRY_EVAL(mh_dyn_link) && test -s conftest.rxlib; then
		LD_RXLIB1="ld -shared"
		RXLIBPRE="lib"
		RXLIBPST=".so"
	else
		mh_dyn_link='ld -G -o conftest.rxlib conftest.o 1>&AC_FD_CC'
		if AC_TRY_EVAL(mh_dyn_link) && test -s conftest.rxlib; then
			LD_RXLIB1="ld -G"
			RXLIBPRE="lib"
			RXLIBPST=".so"
		else
			LD_RXLIB1=""
			RXLIBPRE=""
			RXLIBPST=""
		fi
	fi
fi
rm -f conftest*
])dnl

dnl ---------------------------------------------------------------------------
dnl Determine how to build shared libraries etc..
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_SHARED_LIBRARY],
[
dnl
dnl If compiler is gcc, then flags should be the same for all platforms
dnl (just guessing on this)
dnl
OSAVE=".o.save"
OBJ="o"
EXE=""
GETOPT=""
STATIC_LDFLAGS=""
AIX_DYN="no"
SHLFILE1="rexxsql"
SHLFILE2="rexx$LDB"

AC_REQUIRE([AC_CANONICAL_SYSTEM])
case "$target" in
	*hp-hpux1*)
		SYS_DEFS="-D_HPUX_SOURCE"
		;;
	*hp-hpux*) 
		SYS_DEFS="-D_HPUX_SOURCE"
		;;
	*ibm-aix4*)
		SYS_DEFS="-D_ALL_SOURCE -DAIX"
		AIX_DYN="yes"
		DYN_COMP="-DDYNAMIC"
		;;
	*ibm-aix*)
		SYS_DEFS="-D_ALL_SOURCE -DAIX"
		AIX_DYN="yes"
		DYN_COMP="-DDYNAMIC"
		;;
	*dec-osf*)
		SYS_DEFS="-D_POSIX_SOURCE -D_XOPEN_SOURCE -Olimit 800"
		;;
	*sequent-dynix*)
		;;
	*solaris*)
		;;
	*esix*)
		;;
	sparc*sunos*)
		SYS_DEFS="-DSUNOS -DSUNOS_STRTOD_BUG"
		;;
	*linux*)
		SYS_DEFS="-Wall -ansi -D_POSIX_SOURCE"
		;;
esac

dnl
dnl set variables for compiling if we don't already have it
dnl
if test "$DYN_COMP" = ""; then
AC_MSG_CHECKING(compiler flags for a dynamic object)

cat > conftest.$ac_ext <<EOF
dnl [#]line __oline__ "[$]0"
[#]line __oline__ "configure"
int a=0
EOF

	DYN_COMP=""
	mh_cv_stop=no
	save_cflags="$CFLAGS"
	mh_cv_flags="-fPIC -KPIC +Z"
	for a in $mh_cv_flags; do
		CFLAGS="-c $a"

		mh_compile='${CC-cc} -c $CFLAGS conftest.$ac_ext > conftest.tmp 2>&1'
		if AC_TRY_EVAL(mh_compile); then
			DYN_COMP=""
		else
			slash="\\"
			mh_dyncomp="`egrep -c $slash$a conftest.tmp`"
			if test "$mh_dyncomp" = "0"; then
				DYN_COMP="$a -DDYNAMIC"
				AC_MSG_RESULT($a)
				break
			else
				DYN_COMP=""
			fi
		fi
	done
	if test "$DYN_COMP" = ""; then
		AC_MSG_RESULT(none of $mh_cv_flags supported)
	fi
	CFLAGS=$save_cflags
	rm -f conftest.*
fi

if test "$with_rexx6000" = yes; then
	mh_entry="-eInitFunc"
else
	mh_entry="-bnoentry"
fi

dnl
dnl set variables for linking
dnl
RXPACKEXPORTS1=""
RXPACKEXPORTS2=""
RXLIBPRE="lib"
RXLIBPST=".so"
case "$target" in
	*hp-hpux*) 
		EEXTRA="-Wl,-E"
		LD_RXLIB1="ld -b -q -n"
		RXLIBPST=".sl"
		;;
	*ibm-aix3*)
		STATIC_LDFLAGS="-bnso -bI:/lib/syscalls.exp"
		LD_RXLIB1="ld $mh_entry -bM:SRE -bT:512 -bH:512 -bI:regina.exp"
dnl
dnl Special fix for Object Rexx on AIX
dnl All shared libraries on AIX are named "library.a" EXCEPT Object
dnl Rexx which uses "library.so"
dnl
		if test "$with_orexx" = yes; then
			RXLIBPST=".so"
		else
		RXLIBPST=".a"
		fi
		RXPACKEXPORTS1="-bE:$SHLFILE1.exp"
		RXPACKEXP1="$SHLFILE1.exp"
		RXPACKEXPORTS2="-bE:$SHLFILE2.exp"
		RXPACKEXP2="$SHLFILE2.exp"
		;;
	*ibm-aix4*)
		STATIC_LDFLAGS="-bnso -bI:/lib/syscalls.exp"
		LD_RXLIB1="ld $mh_entry -bM:SRE"
dnl
dnl Special fix for Object Rexx on AIX
dnl All shared libraries on AIX are named "library.a" EXCEPT Object
dnl Rexx which uses "library.so"
dnl
		if test "$with_orexx" = yes; then
			RXLIBPST=".so"
		else
		RXLIBPST=".a"
		fi
		RXPACKEXPORTS1="-bE:$SHLFILE1.exp"
		RXPACKEXP1="$SHLFILE1.exp"
		RXPACKEXPORTS2="-bE:$SHLFILE2.exp"
		RXPACKEXP2="$SHLFILE2.exp"
		;;
	*dec-osf*)
		LD_RXLIB1="ld -shared"
		;;
	*esix*)
		LD_RXLIB1="ld -G"
		DYNAMIC_LDFLAGS=""
		;;
	*dgux*)
		LD_RXLIB1="ld -G"
		DYNAMIC_LDFLAGS=""
		;;
	*solaris*)
		LD_RXLIB1="ld -G"
		DYNAMIC_LDFLAGS=""
		;;
	sparc*sunos*)
		LD_RXLIB1="ld"
		;;
	*freebsd*)
		LD_RXLIB1="ld -Bdynamic -Bshareable"
		;;
	*linux*)
		LD_RXLIB1="ld -shared"
		;;
	*pc-sco3.2*)
		EEXTRA="-b elf"
		LD_RXLIB1="ld -G"
		;;
	*qnx*)
		LD_RXLIB1=""
		RXLIBPRE=""
		RXLIBPST=".lib"
		EEXTRA="-mf -N0x20000 -Q"
		;;
	*)
		MH_HOWTO_DYN_LINK()
		;;
esac

if test "$ac_cv_header_dl_h" = "yes" -o "$ac_cv_header_dlfcn_h" = "yes" -o "$AIX_DYN" = "yes"; then
	if test "$with_regina" = yes -o "$with_orexx" = "yes" -o "$with_rexx6000" = "yes"; then
		SHL_TARGETS="${RXLIBPRE}${SHLFILE1}${RXLIBPST} ${RXLIBPRE}${SHLFILE2}${RXLIBPST}"
	else
		SHL_TARGETS=""
	fi
else
	SHL_TARGETS=""
fi

AC_SUBST(EEXTRA)
AC_SUBST(CEXTRA)
AC_SUBST(OSAVE)
AC_SUBST(OBJ)
AC_SUBST(EXE)
AC_SUBST(GETOPT)
AC_SUBST(DYN_COMP)
AC_SUBST(LIBS)
AC_SUBST(SHLIBS)
AC_SUBST(LD_RXLIB1)
AC_SUBST(RXLIBPRE)
AC_SUBST(RXLIBPST)
AC_SUBST(DYNAMIC_LDFLAGS)
AC_SUBST(STATIC_LDFLAGS)
AC_SUBST(SHL_TARGETS)
AC_SUBST(RXPACKEXPORTS1)
AC_SUBST(RXPACKEXP1)
AC_SUBST(RXPACKEXPORTS2)
AC_SUBST(RXPACKEXP2)
])dnl

dnl ---------------------------------------------------------------------------
dnl Setup all Makefile variables
dnl ---------------------------------------------------------------------------
AC_DEFUN([MH_SETUP_MAKEFILE],
[
case "$dbtouse" in
	oracle)
		DBH="ora"
		DBC="ora"
		LDB="ora"
		BDB="ORA"
		SDB=""
		dbsrc="ora"
		MH_CHECK_ORACLE
		;;
	pgsqlodbc)
		DBH="odbc"
		DBC="cli-odbc"
		LDB="pgodbc"
		BDB="PGODBC"
		SDB="-DPGSQLODBC"
		dbsrc="rexxcli"
		MH_CHECK_PGSQLODBC
		;;
	openlink)
		DBH="odbc"
		DBC="cli-odbc"
		LDB="odbc"
		BDB="ODBC"
		SDB="-DOPENLINK"
		dbsrc="rexxcli"
		MH_CHECK_OPENLINK
		;;
	udbc)
		DBH="udbc"
		DBC="cli-odbc"
		LDB="udbc"
		BDB="UDBC"
		SDB="-DUDBC"
		SDB=""
		dbsrc="rexxcli"
		MH_CHECK_OPENLINK_UDBC
		;;
	intersolv)
		DBH="odbc"
		DBC="cli-odbc"
		LDB="odbc"
		BDB="ODBC"
		SDB="-DINTERSOLV"
		dbsrc="rexxcli"
		;;
	sybase)
		DBH="syb"
		DBC="syb"
		LDB="syb"
		BDB="SYB"
		SDB=""
		dbsrc="syb"
		MH_CHECK_SYBASE
		;;
	sqlanywhere)
		DBH="saw"
		DBC="cli-odbc"
		LDB="saw"
		BDB="SAW"
		SDB=""
		dbsrc="rexxcli"
		MH_CHECK_SQLANYWHERE
		;;
	msql)
		DBH="min"
		DBC="min"
		LDB="min"
		BDB="MIN"
		SDB=""
		dbsrc="min"
		MH_CHECK_MSQL
		;;
	msql2)
		DBH="min2"
		DBC="min2"
		LDB="min"
		BDB="MIN"
		SDB=""
		dbsrc="min"
		MH_CHECK_MSQL2
		;;
	mysql)
		DBH="my"
		DBC="my"
		LDB="my"
		BDB="MY"
		SDB=""
		dbsrc="my"
		MH_CHECK_MYSQL
		;;
	db2)
		DBH="db2"
		DBC="cli-odbc"
		LDB="db2"
		BDB="DB2"
		SDB=""
		dbsrc="rexxcli"
		MH_CHECK_DB2
		;;
	solid)
		DBH="sol"
		DBC="cli-odbc"
		LDB="sol"
		BDB="SOL"
		SDB="-DSS_UNIX -DSOLID_SERVER"
		dbsrc="rexxcli"
		MH_CHECK_SOLID
		;;
	velocis)
		DBH="vel"
		DBC="cli-odbc"
		LDB="vel"
		BDB="VEL"
		SDB="-DVELOCIS"
		dbsrc="rexxcli"
		MH_CHECK_VELOCIS
		;;
	unixodbc)
		DBH="odbc"
		DBC="cli-odbc"
		LDB="odbc"
		BDB="ODBC"
		SDB="-DUNIXODBC"
		dbsrc="rexxcli"
		MH_CHECK_UNIXODBC
		;;
	easysoft_oob)
		DBH="esoob"
		DBC="cli-odbc"
		LDB="esoob"
		BDB="ESOOB"
		SDB="-DEASYSOFT"
		dbsrc="rexxcli"
		MH_CHECK_EASYSOFT_OOB
		;;
	*)
		AC_MSG_ERROR("No database specified to be configured")
		;;
esac
AC_SUBST(DBH)
AC_SUBST(DEFS)
AC_SUBST(DBC)
AC_SUBST(LDB)
AC_SUBST(BDB)
AC_SUBST(SDB)
AC_SUBST(dbsrc)
AC_SUBST(MH_DB_LIB)
AC_SUBST(MH_DB_INC)
AC_SUBST(DBLIB1)
AC_SUBST(DBLIB2)
AC_SUBST(DBLIB3)
AC_SUBST(DBLIB4)
AC_SUBST(INCLUDE_DB_MAKEFILE)
])dnl
