/***********************************************************************
 * util.c - Rexx/SQL utility functions
 ***********************************************************************
 *
 * Rexx/SQL. A Rexx interface to SQL databases.
 * Copyright Mark Hessling, 1996-1999.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to:
 *
 *    The Free Software Foundation, Inc.
 *    675 Mass Ave,
 *    Cambridge, MA 02139 USA.
 *
 *
 * If you make modifications to this software that you feel increases
 * it usefulness for the rest of the community, please email the
 * changes, enhancements, bug fixes as well as any and all ideas to 
 * address below.
 * This software is going to be maintained and enhanced as deemed
 * necessary by the community.
 *
 * Mark Hessling  M.Hessling@qut.edu.au  http://www.lightlink.com/hessling/
 *
 */

static char RCSid[] = "$Id:$";

#include "rxpack.h"

#include "dbdefine.h"
#if defined(UNIX)
# include "dbheader.h"
#endif
#include "rexxsql.h"

extern char FName[40];
extern SQLVAL *opt_tbl;
extern char *column_attribute[];

extern RXSQL_SQLCODE_TYPE  SQLCA_SqlCode;
extern RXSQL_ROWCOUNT_TYPE SQLCA_RowCount;
extern long SQLCA_IntCode;
extern RXSQL_SQLSTATE_TYPE SQLCA_SqlState[SQL_SQLSTATE_SIZE+1];
#if defined(USES_COMPUTE_COUNT)
extern RXSQL_COMPUTECOUNT_TYPE  SQLCA_ComputeCount;
#endif

char *RxPackageName = DLLNAME;

#if !defined(DYNAMIC_LIBRARY) && (defined(USE_WINREXX) || defined(USE_QUERCUS))
RexxExitHandler RxExitHandlerForSayTraceRedirection;
#endif

/*-----------------------------------------------------------------------------
 * Table of REXX/SQL Functions. Used to install/de-install functions.
 *----------------------------------------------------------------------------*/
RexxFunction RxPackageFunctions[] = {
  { (RRFD_ARG0_TYPE)NAME_SQLCONNECT_EXT,    (RRFE_ARG1_TYPE)SQLCONNECT   ,(RRFD_ARG2_TYPE)NAME_SQLCONNECT_INT    , 1  },
  { (RRFD_ARG0_TYPE)NAME_SQLDISCONNECT_EXT, (RRFE_ARG1_TYPE)SQLDISCONNECT,(RRFD_ARG2_TYPE)NAME_SQLDISCONNECT_INT , 1  },
  { (RRFD_ARG0_TYPE)NAME_SQLDEFAULT_EXT,    (RRFE_ARG1_TYPE)SQLDEFAULT   ,(RRFD_ARG2_TYPE)NAME_SQLDEFAULT_INT    , 1  },
  { (RRFD_ARG0_TYPE)NAME_SQLCOMMIT_EXT,     (RRFE_ARG1_TYPE)SQLCOMMIT    ,(RRFD_ARG2_TYPE)NAME_SQLCOMMIT_INT     , 1  },
  { (RRFD_ARG0_TYPE)NAME_SQLROLLBACK_EXT,   (RRFE_ARG1_TYPE)SQLROLLBACK  ,(RRFD_ARG2_TYPE)NAME_SQLROLLBACK_INT   , 1  },
  { (RRFD_ARG0_TYPE)NAME_SQLCOMMAND_EXT,    (RRFE_ARG1_TYPE)SQLCOMMAND   ,(RRFD_ARG2_TYPE)NAME_SQLCOMMAND_INT    , 1  },
  { (RRFD_ARG0_TYPE)NAME_SQLPREPARE_EXT,    (RRFE_ARG1_TYPE)SQLPREPARE   ,(RRFD_ARG2_TYPE)NAME_SQLPREPARE_INT    , 1  },
  { (RRFD_ARG0_TYPE)NAME_SQLDISPOSE_EXT,    (RRFE_ARG1_TYPE)SQLDISPOSE   ,(RRFD_ARG2_TYPE)NAME_SQLDISPOSE_INT    , 1  },
  { (RRFD_ARG0_TYPE)NAME_SQLEXECUTE_EXT,    (RRFE_ARG1_TYPE)SQLEXECUTE   ,(RRFD_ARG2_TYPE)NAME_SQLEXECUTE_INT    , 1  },
  { (RRFD_ARG0_TYPE)NAME_SQLOPEN_EXT,       (RRFE_ARG1_TYPE)SQLOPEN      ,(RRFD_ARG2_TYPE)NAME_SQLOPEN_INT       , 1  },
  { (RRFD_ARG0_TYPE)NAME_SQLCLOSE_EXT,      (RRFE_ARG1_TYPE)SQLCLOSE     ,(RRFD_ARG2_TYPE)NAME_SQLCLOSE_INT      , 1  },
  { (RRFD_ARG0_TYPE)NAME_SQLFETCH_EXT,      (RRFE_ARG1_TYPE)SQLFETCH     ,(RRFD_ARG2_TYPE)NAME_SQLFETCH_INT      , 1  },
  { (RRFD_ARG0_TYPE)NAME_SQLGETDATA_EXT,    (RRFE_ARG1_TYPE)SQLGETDATA   ,(RRFD_ARG2_TYPE)NAME_SQLGETDATA_INT    , 1  },
  { (RRFD_ARG0_TYPE)NAME_SQLVARIABLE_EXT,   (RRFE_ARG1_TYPE)SQLVARIABLE  ,(RRFD_ARG2_TYPE)NAME_SQLVARIABLE_INT   , 1  },
  { (RRFD_ARG0_TYPE)NAME_SQLGETINFO_EXT,    (RRFE_ARG1_TYPE)SQLGETINFO   ,(RRFD_ARG2_TYPE)NAME_SQLGETINFO_INT    , 1  },
  { (RRFD_ARG0_TYPE)NAME_SQLDESCRIBE_EXT,   (RRFE_ARG1_TYPE)SQLDESCRIBE  ,(RRFD_ARG2_TYPE)NAME_SQLDESCRIBE_INT   , 1  },
  { (RRFD_ARG0_TYPE)NAME_SQLDROPFUNCS_EXT,  (RRFE_ARG1_TYPE)SQLDROPFUNCS ,(RRFD_ARG2_TYPE)NAME_SQLDROPFUNCS_INT  , 1  },
  { (RRFD_ARG0_TYPE)NAME_SQLLOADFUNCS_EXT,  (RRFE_ARG1_TYPE)SQLLOADFUNCS ,(RRFD_ARG2_TYPE)NAME_SQLLOADFUNCS_INT  , 0  }, /* don't load this for DLL */
  { NULL,                                   NULL                         ,NULL                                   , 0  }
};

/*
 * This variable is used to determine if the dynamic library version
 * of Rexx/SQL has been initialised.  It is used to run InitRexxSQL()
 * after the DLL has been loaded, but SQLLoadFuncs() is not called
 * again.
 */
ULONG RexxSQLInitialised = 0;

/*
 * Settable variables
 * 
 * ---------------------------------------------------------------------
 * Limits number of rows returned by SQLCOMMAND(). If set to zero (0) then
 * all rows in query are returned (unless memory is exhausted in which case
 * an error is returned)!
 */
ULONG RowLimit =  0;	/* Default is unlimited! */

/* ---------------------------------------------------------------------
 * Limits size of a "LONG" column to the specified size.
 * This limit is only imposed by a normal row FETCH, and is ignored by
 * the size specified in SqlGetData() 
 */
ULONG LongLimit = DEFAULT_LONG_LIMIT;	/* Default is 32k */

/* ---------------------------------------------------------------------
 * If TRUE, the SQL statement is saved on each parse (SqlCommand()
 * & SqlPrepare()).  If TRUE then on errors, the last SQL statement can be
 * retrieved in a REXX program by calling SQLVARIABLE().
 * eg.      error_stmt = SqlVariable('sqltext');
 */
ULONG SaveSQL = TRUE;	/* Default is to save each SQL statement! */

/* ---------------------------------------------------------------------
 * If TRUE, the default AUTOCOMMIT behaviour for statements is ON.
 */
ULONG AutoCommit = FALSE;	/* Default is to have AUTOCOMMIT OFF! */

/* ---------------------------------------------------------------------
 * If TRUE, any truncation errors will be ignored
 */
ULONG IgnoreTruncate = FALSE;	/* Default is to fail on truncate errors */

/* ---------------------------------------------------------------------
 * If TRUE, REXX/SQL will allow the user to supply ? as placemarkers
 * instead of :1 or :name placemarkers. Only applicable to Oracle.
 */
ULONG StandardPlacemarkers = FALSE;	/* Default is to NOT substitute ? */

/* ---------------------------------------------------------------------
 * If TRUE, the database supports transactions.
 */
ULONG SupportsTransactions = SUPPORTSTRANSACTIONS;	/* Default database dependent */

/* ---------------------------------------------------------------------
 * If TRUE, the database supports placemarkers.
 */
ULONG SupportsPlacemarkers = SUPPORTSPLACEMARKERS;	/* Default database dependent */

/* ---------------------------------------------------------------------
 * If TRUE, the database sets SQLCA.ROWCOUNT for DML statements
 */
ULONG SupportsDMLRowcount = SUPPORTSDMLROWCOUNT;	/* Default database dependent */

/* ---------------------------------------------------------------------
 * If TRUE, the database supports the SqlGetData() function
 */
ULONG SupportsSqlGetData = SUPPORTSSQLGETDATA;	/* Default database dependent */

/* ---------------------------------------------------------------------
 * String representing a NULL value returned from a SELECT statement.
 * Default is the empty string; ""
 */
RXSTRING RXSNullStringOut;
char NullStringOut[MAX_IDENTIFIER+1]; /* set in InstallSQLVariables() */

/* ---------------------------------------------------------------------
 * String representing a NULL value when passed as a bind value.
 * Default is ""
 */
RXSTRING RXSNullStringIn;
char NullStringIn[MAX_IDENTIFIER+1]; /* set in InstallSQLVariables() */

#if !defined(HAVE_STRERROR)
/*
 * This function and the following description borrowed from Regina 0.08a
 *
 * Sigh! This must probably be done this way, although it's incredibly 
 * backwards. Some versions of gcc comes with a complete set of ANSI C
 * include files, which contains the definition of strerror(). However,
 * that function does not exist in the default libraries of SunOS. 
 * To circumvent that problem, strerror() is #define'd to get_sys_errlist() 
 * in config.h, and here follows the definition of that function. 
 * Originally, strerror() was #defined to sys_errlist[x], but that does
 * not work if string.h contains a declaration of the (non-existing) 
 * function strerror(). 
 *
 * So, this is a mismatch between the include files and the library, and
 * it should not create problems for Regina. However, the _user_ will not
 * encounter any problems until he compiles Regina, so we'll have to 
 * clean up after a buggy installation of the C compiler!
 */
char *rxsql_get_sys_errlist( int num )
{
   extern char *sys_errlist[] ;
   return sys_errlist[num] ;
}
#endif

/*-----------------------------------------------------------------------------
 * Parse the supplied statement and return 1 if the first word of the
 * statement is "select" (any case) or 0 otherwise.
 *----------------------------------------------------------------------------*/
int SelectStatement

#ifdef HAVE_PROTO
    (char *stmt,long len)
#else
    (stmt,len)
    char    *stmt;
    long    len;
#endif

{
 char buf[7];
 char *p=stmt;
 int i=0;

 InternalTrace("SelectStatement", "%s,%d", stmt, len);
 /* Remove leading whitespace */
 while (len && isspace(*p)) 
 {
    p++;
    len--;
 }
 if (len<6L)
    return(0);
 buf[6] = '\0';

 for(i=0;i<6;i++)
 {
    buf[i] = (islower(*p)) ? toupper(*p) : *p;
    p++;
 }
 if (strcmp(buf,"SELECT") == 0)
    return(1);
 return(0);
}

/*-----------------------------------------------------------------------------
 * Store the error message for INTERNAL errors in SQLCA.SQLTEXT variable.
 *----------------------------------------------------------------------------*/
int SetIntError

#ifdef HAVE_PROTO
    (char *fn, int lineno, int errcode, char *errmsg)
#else
    (fn,lineno,errcode,errmsg)
    char *fn;
    int   lineno;
    int   errcode;
    char *errmsg;
#endif

{
 char  msg[MAX_ERROR_TEXT];

 InternalTrace("SetIntError", "%s,%d,%d,%s", fn, lineno, errcode, errmsg );

 SQLCA_IntCode = -errcode;

 /* 
  * Set SQLCA.INTERRM variable
  */
 if (RxGetRunFlags() == 10)
    (void)sprintf(msg, "REXX/SQL-%02d: %s [%s:%d]", errcode, errmsg, fn, lineno);
 else
    (void)sprintf(msg, "REXX/SQL-%02d: %s", errcode, errmsg);
 (void)SetRexxVariable(SQLCA_INTERRM, strlen(SQLCA_INTERRM),msg, strlen(msg));

 /* 
  * Set SQLCA.INTCODE variable 
  */
 (void)sprintf(msg, "%ld", SQLCA_IntCode);
 (void)SetRexxVariable(SQLCA_INTCODE, strlen(SQLCA_INTCODE),msg, strlen(msg));

 return(SQLCA_IntCode);
}

/*-----------------------------------------------------------------------------
 * Set elements of the compound variable "SQLCA.".  SQLCA. holds the results of
 * any call to REXXSQL. It should never be set by the REXX program!
 *----------------------------------------------------------------------------*/
void SetSQLCA

#ifdef HAVE_PROTO
    (RXSQL_SQLCODE_TYPE sqlcode, RXSQL_SQLSTATE_TYPE *sqlstate, RXSQL_SQLERRM_TYPE *errm, char *txt)
#else
    (sqlcode, sqlstate, errm, txt)
    RXSQL_SQLCODE_TYPE   sqlcode;
    RXSQL_SQLSTATE_TYPE  *sqlstate;
    RXSQL_SQLERRM_TYPE   *errm;
    char   *txt;
#endif

{
 char    buf[50];

 InternalTrace("SetSQLCA", "%d,%s,%s,%s", sqlcode, sqlstate, errm, txt);

 SQLCA_SqlCode = sqlcode;
 strcpy( SQLCA_SqlState, sqlstate );

 (void)sprintf(buf, "%ld", sqlcode);
 (void)SetRexxVariable(SQLCA_SQLCODE,  strlen(SQLCA_SQLCODE),buf, strlen(buf));
 (void)SetRexxVariable(SQLCA_SQLERRM,  strlen(SQLCA_SQLERRM),errm, strlen(errm));
 (void)SetRexxVariable(SQLCA_SQLTEXT,  strlen(SQLCA_SQLTEXT),txt, strlen(txt));
 (void)SetRexxVariable(SQLCA_SQLSTATE, strlen(SQLCA_SQLSTATE),sqlstate, strlen(sqlstate));

 if (!(sqlcode==0 && strcmp(sqlstate,"")==0 && strcmp(errm,"")==0))
   {
    (void)SetRexxVariable(SQLCA_INTCODE, strlen(SQLCA_INTCODE),"-1", 2);
    (void)sprintf(buf, "REXX/SQL-1: Database Error");
    (void)SetRexxVariable(SQLCA_INTERRM, strlen(SQLCA_INTERRM),buf, strlen(buf));
   }
 return;
}

/*-----------------------------------------------------------------------------
 * Set RowCount and put into REXX "SQLCA.ROWCOUNT" compound variable.
 *----------------------------------------------------------------------------*/
void SetRowCount

#ifdef HAVE_PROTO
    (RXSQL_ROWCOUNT_TYPE rowcount)
#else
    (rowcount)
    RXSQL_ROWCOUNT_TYPE rowcount;
#endif

{
 char    buf[50];

 InternalTrace("SetRowCount", "%d", rowcount);

 if (SQLCA_RowCount != rowcount) 
   {
    SQLCA_RowCount = rowcount;
    (void)sprintf(buf, "%lu", rowcount);
    (void)SetRexxVariable(SQLCA_ROWCOUNT,  strlen(SQLCA_ROWCOUNT),buf, strlen(buf));
   }
 return;
}

#if defined(USES_COMPUTE_COUNT)
/*-----------------------------------------------------------------------------
 * Set ComputeCount and put into REXX "SQLCA.COMPUTECOUNT" compound variable.
 *----------------------------------------------------------------------------*/
void SetComputeCount

#ifdef HAVE_PROTO
    (RXSQL_COMPUTECOUNT_TYPE computecount)
#else
    (computecount)
    RXSQL_COMPUTECOUNT_TYPE computecount;
#endif

{
 char    buf[50];

 InternalTrace("SetComputeCount", "%d", computecount );

 if (SQLCA_ComputeCount != computecount) 
   {
    SQLCA_ComputeCount = computecount;
    (void)sprintf(buf, "%lu", computecount);
    (void)SetRexxVariable(SQLCA_COMPUTECOUNT,  strlen(SQLCA_COMPUTECOUNT),buf, strlen(buf));
   }
 return;
}
#endif

/*-----------------------------------------------------------------------------
 * Clear the error code, error message etc.
 *----------------------------------------------------------------------------*/
void ClearDBError

#ifdef HAVE_PROTO
    (void)
#else
    ()
#endif

{
#if defined(USES_MULTIPLE_ERRORS)
 char tmp[50];
#endif

 InternalTrace("ClearDBError", NULL);

 SetRowCount(0L);
#if defined(USES_COMPUTE_COUNT)
 SetComputeCount(0L);
#endif
#if defined(USES_MULTIPLE_ERRORS)
 sprintf(tmp,"%s.0",SQLCA_SQLERRM);
 (void)SetRexxVariable(tmp, strlen(tmp), "0", 1);
 sprintf(tmp,"%s.0",SQLCA_SQLSTATE);
 (void)SetRexxVariable(tmp, strlen(tmp), "0", 1);
#endif
 SetSQLCA(0L, "", "", "");
 return;
}

/*-----------------------------------------------------------------------------
 * Clear the internal error code, error message etc.
 *----------------------------------------------------------------------------*/
void ClearIntError

#ifdef HAVE_PROTO
    (void)
#else
    ()
#endif

{
 SQLCA_IntCode = 0L;

 InternalTrace("ClearIntError", NULL );

 /* 
  * Set SQLCA.INTERRM variable 
  */
 (void)SetRexxVariable(SQLCA_INTERRM, strlen(SQLCA_INTERRM),"", 0);

 /* 
  * Set SQLCA.INTCODE variable 
  */
 (void)SetRexxVariable(SQLCA_INTCODE, strlen(SQLCA_INTCODE),"0", 1);

 return;
}

/*-----------------------------------------------------------------------------
 * Copy a string "s" to the RXSTRING structure pointed to by "target".
 * The string is not null terminated (unless the null is included as the
 * last byte of the string and is counted in "len".
 *----------------------------------------------------------------------------*/
ULONG PutString

#ifdef HAVE_PROTO
    (RXSTRING *target, char *s, size_t len)
#else
    (target, s, len)
    RXSTRING	*target;
    char	*s;
    size_t      len;
#endif

{
 char        *t=NULL;

 if (len > RXAUTOBUFLEN)
   {
#if defined(WIN32) && (defined(USE_OREXX) || defined(USE_WINREXX) || defined(USE_QUERCUS))
    if ((t = (char*)GlobalLock(GlobalAlloc(GMEM_FIXED,len+1))) == (char*)NULL)
#elif defined(USE_OS2REXX)
    if ((BOOL)DosAllocMem( (void**)&t, len+1, fPERM|PAG_COMMIT))
#else
    if ((t = (char*)malloc(len+1)) == (char*)NULL)
#endif
    if ((t = (char*)malloc(len+1)) == (char*)NULL)
       return 1;
    target->strptr = (RXSTRING_STRPTR_TYPE)t;
   }

 memcpy(target->strptr, s, len);
 target->strlength = len;
 return 0;
}


/*-----------------------------------------------------------------------------
 * Return an error to REXX interpreter.
 *----------------------------------------------------------------------------*/
ULONG ReturnError

#ifdef HAVE_PROTO
    (RXSTRING *retptr, int errcode, char *errmsg)
#else
    (retptr, errcode, errmsg)
    RXSTRING	*retptr;
    int         errcode;
    char	*errmsg;
#endif

{
 SetIntError( __FILE__, __LINE__, errcode, errmsg);
 return RxReturnNumber(retptr, -errcode);
}




/*-----------------------------------------------------------------------------
 * Makes an identifier (for a Connection, Stem or Statement) from a given
 * string. Leading and trailing whitespace is removed and the string is
 * capitalised. The variable must be the following regular expression:
 *   [A-Za-z!?][A-Za-z0-9!?_\.]*
 * That is the variable :-
 *   (i) The variable name must start with an alpha character or '!' or '?'.
 *  (ii) The rest of the variable name must be one of the above characters or
 *       a decimal digit or underscore or period.
 *----------------------------------------------------------------------------*/
int MkIdentifier

#ifdef HAVE_PROTO
    (RXSTRING var, PSZ buf, size_t buflen)
#else
    (var, buf, buflen)
    RXSTRING  var;
    PSZ       buf;
    size_t    buflen;
#endif

{
 char      *p = RXSTRPTR(var);
 size_t    len = RXSTRLEN(var);
 size_t    cnt = 0;

 if (len == 0)
    return(SetIntError( __FILE__, __LINE__, 51, "zero length identifier"));

 /* 
  * Make room for the terminating byte. 
  */
 buflen--;

 /* 
  * Remove leading whitespace 
  */
 while (len && isspace(*p)) 
   {
    p++;
    len--;
   }

 /* 
  * Special check for 1st character 
  */
 if (len && (isalpha(*p) || *p == '!' || *p == '?')) 
   {
    *buf++ = (islower(*p)) ? toupper(*p) : *p;
    p++;
    len--;
   }

 /* 
  * Copy identifier to destination 
  */
 while (len && (isalnum(*p) || *p == '_' || *p == '!' || *p == '?' || *p== '.'))
   {
    *buf = (islower(*p)) ? toupper(*p) : *p;
    p++;
    len--;
    if (++cnt <= buflen) 
       buf++;
   }

 /* 
  * Remove trailing whitespace 
  */
 while (len && isspace(*p)) 
   {
    p++;
    len--;
   }

 /* 
  * Check for garbage 
  */
 if (len)
    return(SetIntError( __FILE__, __LINE__, 52, "garbage in identifier name" ));

 *buf = '\0';
 return 0;
}

/*-----------------------------------------------------------------------------
 * Save SQL statement in cursor statement buffer.  Dynamically extends the
 * statement buffer when required.
 *----------------------------------------------------------------------------*/
int SaveSqlStatement

#ifdef HAVE_PROTO
    (char **sql_stmt, int *sql_stmt_sz, char *statement, int length)
#else
    (sql_stmt, sql_stmt_sz, statement, length)
    char    **sql_stmt;
    int     *sql_stmt_sz;
    char    *statement;
    int     length;
#endif

{
 int len=length+1;
 char *ptr=NULL;

 InternalTrace("SaveSqlStatement", "%x,%x,%s,%d", sql_stmt, sql_stmt_sz, statement, length );

 if (len > *sql_stmt_sz)
   {
    /* New statement size is > than previous */
    if (*sql_stmt != (char*)NULL)
      {
       /* Free old statement */
       free(*sql_stmt); 
       *sql_stmt = (char*)NULL;
       *sql_stmt_sz = 0;
      }

    /* Allocate a buffer for the new statement */
    if ((*sql_stmt = (char*)malloc(len)) == (char*)NULL)
       return(SetIntError( __FILE__, __LINE__, 10, "out of memory"));
    *sql_stmt_sz = len;
   }

 /* Save the statement */
 (void)memcpy(*sql_stmt, statement, length);
 ptr=*sql_stmt;
 ptr[length] = '\0';
 return 0;
}

/*-----------------------------------------------------------------------------
 * Count the number of '?' in a SQL statement. This is a kludge for ODBC
 * drivers that don't support SQLNumParams.
 *----------------------------------------------------------------------------*/
int CountQuestionMarks

#ifdef HAVE_PROTO
    (char *sql_stmt)
#else
    (sql_stmt)
    char    *sql_stmt;
#endif

{
 int num=0;

 InternalTrace("CountQuestionMarks", "%s", sql_stmt );

 while(*sql_stmt++)
 {
    if (*sql_stmt == '?')
       num++;
 }
 return num;
}

/*-----------------------------------------------------------------------------
 * Install a REXX/SQL variable structure.
 *----------------------------------------------------------------------------*/
SQLVAL *InstallSQLVariable

#ifdef HAVE_PROTO
    (int *rc,char *name, SQLVAL *first, int upd, int dtype, int max, int option, void *val, int valtype)
#else
    (rc,name, first, upd, dtype, max, option, val, valtype)
    int    *rc;
    char   *name;
    SQLVAL *first;
    int    upd;
    int    dtype;
    int    max;
    int    option;
    void   *val;
    int    valtype;
#endif

{
 SQLVAL  *opt=NULL;

 InternalTrace("InstallSQLVariable", "%x,%s,%x,%d,%d,%d,%d,%x,%d", rc,name, first, upd, dtype, max, option, val, valtype );

 if ((opt = (SQLVAL*)ll_add(first,first)) == (SQLVAL*)NULL)
 {
    *rc = SetIntError( __FILE__, __LINE__, 10, "out of memory");
    return(NULL);
 }
 (void)strcpy(opt->name, name);
 opt->user_update = (char)upd;
 opt->dtype = (char)dtype;
 opt->maxlen = max;
 opt->value = val;
 opt->option = option;
 opt->valtype = valtype;
 *rc = 0;
 return(opt);
}

/*-----------------------------------------------------------------------------
 * This function converts a REXX array into an array of argv's
 *----------------------------------------------------------------------------*/
int ArrayToArgv

#ifdef HAVE_PROTO
    (RXSTRING *names, RXSTRING *values, ULONG *Argc, RXSTRING *Argv)
#else
    (names, values, Argc, Argv)
    RXSTRING *names;
    RXSTRING *values;
    ULONG *Argc;
    RXSTRING *Argv;
#endif

{
 int i=0,num_args=1;
 RXSTRING counter;
 char *new_name=NULL;
 char *new_value=NULL;
 int num_names=0,num_values=0;

 InternalTrace("ArrayToArgv", "%x,%x,%x,%x", names, values, Argc, Argv );

 if ((new_name = (char *)malloc(RXSTRLEN(*names)+1)) == NULL)
    return(SetIntError( __FILE__, __LINE__, 10, "out of memory"));
 memcpy(new_name,RXSTRPTR(*names),RXSTRLEN(*names));
/*
 if ((counter.strptr = (char *)malloc(50+1)) == NULL)
    return(SetIntError( __FILE__, __LINE__, 10, "out of memory"));
 counter.strlength = 50;
*/
 new_name[RXSTRLEN(*names)-1] = '\0';
 if ((GetRexxVariable(new_name,&counter,0)) == NULL)
 {
    free(new_name);
    free(counter.strptr);
    return(SetIntError( __FILE__, __LINE__, 10, "rexx variable not found"));
 }
 num_names = atoi(RXSTRPTR(counter));

 if (values != NULL)
 {
    num_args = 2;
    if ((new_value = (char *)malloc(RXSTRLEN(*values)+1)) == NULL)
    {
       free(new_name);
       free(counter.strptr);
       return(SetIntError( __FILE__, __LINE__, 10, "out of memory"));
    }
    memcpy(new_value,RXSTRPTR(*values),RXSTRLEN(*values));
    new_value[RXSTRLEN(*values)-1] = '\0';
    if ((GetRexxVariable(new_value,&counter,0)) == NULL)
    {
       free(new_name);
       free(counter.strptr);
       return(SetIntError( __FILE__, __LINE__, 10, "out of memory"));
    }
    num_values = atoi(RXSTRPTR(counter));
    if (num_names != num_values)
    {
       free(new_name);
       free(counter.strptr);
       return(SetIntError( __FILE__, __LINE__, 88, "array index counts not equal"));
    }
 }
 free(counter.strptr);

 for (i=1;i<=num_names;i++)
 {
    if ((GetRexxVariable(new_name,Argv,i)) == NULL)
    {
       free(new_name);
       return(SetIntError( __FILE__, __LINE__, 10, "out of memory"));
    }
    Argv++;
    if (values != NULL)
    {
       if ((GetRexxVariable(new_value,Argv,i)) == NULL)
       {
          free(new_name);
          return(SetIntError( __FILE__, __LINE__, 10, "out of memory"));
       }
       Argv++;
    }
 }
 free(new_name);
 free(new_value);
 *Argc = num_names*num_args;
 return 0;
}

/*-----------------------------------------------------------------------------
 * This function copies an array of arvs to another
 *----------------------------------------------------------------------------*/
int ArgvToArgv

#ifdef HAVE_PROTO
    (ULONG argc, RXSTRING *argv, ULONG *Argc, RXSTRING *Argv)
#else
    (argc, argv, Argc, Argv)
    ULONG argc;
    RXSTRING *argv;
    ULONG *Argc;
    RXSTRING *Argv;
#endif

{
 ULONG i=0L;

 InternalTrace("ArgvToArgv","%d,%x,%x,%x", argc, argv, Argc, Argv );

 for (i=0;i<argc;i++)
   {
    if ((Argv[i].strptr = (char *)malloc(RXSTRLEN(argv[i])+1)) == NULL)
       return(SetIntError( __FILE__, __LINE__, 10, "out of memory"));
    memcpy(RXSTRPTR(Argv[i]),RXSTRPTR(argv[i]),RXSTRLEN(argv[i]));
    Argv[i].strptr[RXSTRLEN(argv[i])] = '\0';
    Argv[i].strlength = RXSTRLEN(argv[i]);
   }
 *Argc = argc;
 return 0;
}

/*-----------------------------------------------------------------------------
 * This function frees an array of argvs
 *----------------------------------------------------------------------------*/
RXSTRING *FreeArgv

#ifdef HAVE_PROTO
    (ULONG *Argc, RXSTRING *Argv)
#else
    (Argc,Argv)
    ULONG *Argc;
    RXSTRING *Argv;
#endif

{
 int i=0;

 InternalTrace("FreeArgv", "%x,%x", Argc,Argv );

 if (*Argc == 0
 || Argv == NULL)
    return(NULL);

 for (i=0;i<MAX_BINDVARS*2;i++)
   {
    if (RXSTRPTR(Argv[i]) != NULL)
       free(Argv[i].strptr);
    Argv[i].strptr = NULL;
    Argv[i].strlength = 0;
   }
 *Argc = 0;
 free(Argv);
 return(NULL);
}


/*-----------------------------------------------------------------------------
 * This function is called to initiate REXX/SQL interface.
 *----------------------------------------------------------------------------*/
int InitRexxSQL

#ifdef HAVE_PROTO
    (char *progname,int regfunc)
#else
    (progname,regfunc)
    char *progname;
    int regfunc;
#endif

{
 ULONG rc=0L;

 InternalTrace("InitRexxSQL", "%s,%d", progname, regfunc );

 /* 
  * Install REXX/SQL variable descriptors 
  */
 if ((rc = InstallSQLVariables()))
     return(rc);
 
 /*
  * Call any database-specific initialisation function here
  */
 if ((rc = DBInitialise()))
    return(rc);

 RexxSQLInitialised = MAGIC_NUMBER;

 return 0;
}


/*-----------------------------------------------------------------------------
 * This function is called to terminate all activity with REXX/SQL.
 *----------------------------------------------------------------------------*/
int TerminateRexxSQL

#ifdef HAVE_PROTO
    (char *progname)
#else
    (progname)
    char *progname;
#endif

{
 int rc=0;

 FunctionPrologue(progname, 0L, NULL);

 /* Release the REXX/SQL environment. */
 if ((rc = ReleaseDbEnvironment()))
    return rc;

 opt_tbl = ll_free(opt_tbl);

 return 0;
}

/*-----------------------------------------------------------------------------
 * Add a SQLVAL structure to a linked list.
 *----------------------------------------------------------------------------*/
SQLVAL *ll_add

#ifdef HAVE_PROTO
    (SQLVAL *first,SQLVAL *curr)
#else
    (first,curr)
    SQLVAL *first;
    SQLVAL *curr;
#endif
{
 SQLVAL *next=NULL;
 if ((next=(SQLVAL *)malloc(sizeof(SQLVAL))) != (SQLVAL *)NULL)
   {
    if (curr == NULL)
      {
       first = next;
       next->next = NULL;
      }
    else
      {
       if (curr->next != NULL)
          curr->next->prev = next;
       next->next = curr->next;
       curr->next = next;
      }
    next->prev = curr;
   }
 next->value = NULL;
 return(next);
}

/*-----------------------------------------------------------------------------
 * Free a SQLVAL linked list.
 *----------------------------------------------------------------------------*/
SQLVAL *ll_free

#ifdef HAVE_PROTO
    (SQLVAL *first)
#else
    (first)
    SQLVAL *first;
#endif
{
 SQLVAL *curr=first;
 SQLVAL *new_curr=NULL;
 RXSTRING *rxs=NULL;
 RXSTRING_DT *rxs_dt=NULL;
 int i=0;

 while (curr != NULL)
   {
    if ((curr->option)&OPT_DYNAMIC)
      {
       switch(curr->dtype)
         {
          case TYPE_INT:
          case TYPE_BOOL:
               if (curr->value)
                  free(curr->value);
               break;
          case TYPE_STRING:
               rxs =(RXSTRING*)(curr->value);
               if (rxs)
                 {
                  if (rxs->strptr) free(rxs->strptr);
                  free(rxs);
                 }
               break;
          case TYPE_ARRAY:
               rxs =(RXSTRING*)(curr->value);
               if (rxs)
                 {
                  for (i=0;;i++)
                    {
                     if (rxs[i].strlength == 0)
                        break;
                     if (rxs[i].strptr) free(rxs[i].strptr);
                    }
                  free(rxs);
                 }
               break;
          case TYPE_DATATYPE:
               rxs_dt =(RXSTRING_DT*)(curr->value);
               if (rxs_dt)
                 {
                  for (i=0;;i++)
                     {
                      if (rxs_dt[i].strlength == 0)
                         break;
                      if (rxs_dt[i].strptr) free(rxs_dt[i].strptr);
                     }
                 }
               free(rxs_dt);
               break;
         }
      }
    new_curr = curr->next;
    free(curr);
    curr = new_curr;
   }
 return((SQLVAL *)NULL);
}

/*-----------------------------------------------------------------------------
 * Find an entry in a SQLVAL linked list.
 *----------------------------------------------------------------------------*/
SQLVAL *ll_find

#ifdef HAVE_PROTO
    (SQLVAL *first, char *name)
#else
    (first,name)
    SQLVAL *first;
    char *name;
#endif
{
 SQLVAL *curr=NULL;
 curr = first;
 while (curr != NULL)
   {
    if (strcmp(curr->name,name) == 0)
      {
       return(curr);
      }
    curr = curr->next;
   }
 return(NULL);
}

/*-----------------------------------------------------------------------------
 * Print a usage message.
 *----------------------------------------------------------------------------*/
void usage

#ifdef HAVE_PROTO
    (void)
#else
    ()
#endif

{
    (void)fprintf(stderr,
                  "\nVersion: %s %s %s %s %s\n\nUsage:   %s [-h]\n         %s [-idv] [REXX-script-name] [arguments...]\n\n",
                  RxPackageName,
                  REXXSQL_VERSION,
                  REXXSQL_DATE,
                  CURRENT_OS,
                  CURRENT_DB,
                  RxPackageName,
                  RxPackageName);
    exit(BAD_ARGS);
}

/*-----------------------------------------------------------------------------
 * This function is used to initialise the external functions from
 * within a dynamic library when the SysAddFunc() REXX function
 * is called.
 *----------------------------------------------------------------------------*/
#if defined(USE_REXX6000)
USHORT InitFunc(RXFUNCBLOCK **FuncBlock)
{
 static RXFUNCBLOCK funcarray[] =
  {
  { NAME_SQLCONNECT_EXT,    SQLCONNECT   ,NULL },
  { NAME_SQLDISCONNECT_EXT, SQLDISCONNECT,NULL },
  { NAME_SQLDEFAULT_EXT,    SQLDEFAULT   ,NULL },
  { NAME_SQLCOMMIT_EXT,     SQLCOMMIT    ,NULL },
  { NAME_SQLROLLBACK_EXT,   SQLROLLBACK  ,NULL },
  { NAME_SQLCOMMAND_EXT,    SQLCOMMAND   ,NULL },
  { NAME_SQLPREPARE_EXT,    SQLPREPARE   ,NULL },
  { NAME_SQLDISPOSE_EXT,    SQLDISPOSE   ,NULL },
  { NAME_SQLEXECUTE_EXT,    SQLEXECUTE   ,NULL },
  { NAME_SQLOPEN_EXT,       SQLOPEN      ,NULL },
  { NAME_SQLCLOSE_EXT,      SQLCLOSE     ,NULL },
  { NAME_SQLFETCH_EXT,      SQLFETCH     ,NULL },
  { NAME_SQLGETDATA_EXT     SQLGETDATA   ,NULL },
  { NAME_SQLVARIABLE_EXT,   SQLVARIABLE  ,NULL },
  { NAME_SQLGETINFO_EXT,    SQLGETINFO   ,NULL },
  { NAME_SQLDESCRIBE_EXT,   SQLDESCRIBE  ,NULL },
  { NAME_SQLDROPFUNCS_EXT,  SQLDROPFUNCS ,NULL },
  { NULL,                   NULL,         NULL }
  };
 *FuncBlock = funcarray;
 return (USHORT)0;
}
#endif

int InitialisePackage

#ifdef HAVE_PROTO
   ( void )
#else
   ( )
#endif
{
   int rc = 0;

   RowLimit= 0L ;
   LongLimit = DEFAULT_LONG_LIMIT;
   SaveSQL = TRUE;
   AutoCommit = FALSE;
   IgnoreTruncate = FALSE;
   StandardPlacemarkers = FALSE;
   SupportsTransactions = SUPPORTSTRANSACTIONS;
   SupportsPlacemarkers = SUPPORTSPLACEMARKERS;
   SupportsDMLRowcount  = SUPPORTSDMLROWCOUNT;
   SupportsSqlGetData   = SUPPORTSSQLGETDATA;
   SQLCA_SqlCode = (-1L);
   SQLCA_RowCount = (-1L);
   SQLCA_IntCode = (-1L);
   rc = InitRexxSQL( RxPackageName, 0 );
   return rc;
}

int TerminatePackage

#ifdef HAVE_PROTO
   ( void )
#else
   ( )
#endif
{
   TerminateRexxSQL( RxPackageName );
   return 0;
}

#if !defined(DYNAMIC_LIBRARY) && (defined(USE_WINREXX) || defined(USE_QUERCUS))
# if defined(USE_WINREXX)
int RXSQL_APIENTRY RxExitHandlerForSayTraceRedirection
   ( int ExitNumber, int Subfunction, PEXIT ParmBlock )
# else
LONG RxExitHandlerForSayTraceRedirection
   ( LONG ExitNumber, LONG Subfunction, PEXIT ParmBlock )
# endif
{
   long i = 0;
   int rc = 0;

   switch( Subfunction )
   {
      case RXSIOSAY:
      {
         RXSIOSAY_PARM *say_parm = (RXSIOSAY_PARM *)ParmBlock;
         if ( say_parm->rxsio_string.strptr != NULL )
         {
            for( i = 0; i < (long)say_parm->rxsio_string.strlength; i++ )
               fputc( ( char )say_parm->rxsio_string.strptr[i], stdout );
         }
         fputc( '\n', stdout );
         rc = RXEXIT_HANDLED;
         break;
      }
      case RXSIOTRC:
      {
         RXSIOTRC_PARM *trc_parm = (RXSIOTRC_PARM *)ParmBlock;
         if ( trc_parm->rxsio_string.strptr != NULL )
         {
            for( i = 0; i < (long)trc_parm->rxsio_string.strlength; i++ )
               fputc( ( char )trc_parm->rxsio_string.strptr[i], stdout );
         }
         fputc( '\n', stdout );
         rc = RXEXIT_HANDLED;
         break;
      }
      case RXSIOTRD:
      {
         RXSIOTRD_PARM *trd_parm = (RXSIOTRD_PARM *)ParmBlock;
         int i = 0, ch = 0;
         do
         {
            if ( i < 256 )
               trd_parm->rxsiotrd_retc.strptr[i++] = ch = getc( stdin ) ;
         } while( ch != '\012' && (ch != EOF ) ) ;
         trd_parm->rxsiotrd_retc.strlength = i - 1;
         rc = RXEXIT_HANDLED;
         break;
      }
      case RXSIODTR:
      {
         RXSIODTR_PARM *dtr_parm = (RXSIODTR_PARM *)ParmBlock;
         int i = 0, ch = 0;
         do
         {
            if ( i < 256 )
               dtr_parm->rxsiodtr_retc.strptr[i++] = ch = getc( stdin ) ;
         } while( ch != '\012' && (ch != EOF ) ) ;
         dtr_parm->rxsiodtr_retc.strlength = i - 1;
         rc = RXEXIT_HANDLED;
         break;
      }
         break;
      default:
         rc = RXEXIT_NOT_HANDLED;
         break;
   }
  return rc;
}
#endif

/*
 * Because WinRexx and Personal Rexx load and unload the Rexx/SQL DLL
 * each time an external function is called, we need to ensure that
 * the DLL stays loaded in memory, so that memory we have initialised
 * stays set.
 * Why they do this is beyond me :-(
 */
#if defined(DYNAMIC_LIBRARY) && (defined(USE_WINREXX) || defined(USE_QUERCUS))
BOOL WINAPI DllMain( HINSTANCE hDLL, DWORD dwReason, LPVOID pReserved)
{
   switch( dwReason)
   {
      case DLL_PROCESS_ATTACH:
         LoadLibrary( RxPackageName );
         break;
      case DLL_PROCESS_DETACH:
         FreeLibrary( GetModuleHandle( RxPackageName ) );
         break;
      case DLL_THREAD_ATTACH:
         break;
      case DLL_THREAD_DETACH:
         break;
   }
   return(TRUE);
}
#endif
